#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
.. currentmodule:: normanpg.version
.. moduleauthor:: Pat Daburu <pat@daburu.net>

This module contains project version information.
"""

__version__ = '0.0.4'  #: the working version
__release__ = '0.0.4'  #: the release version
